-- LUA_MONITOR
-- BLUA remakes of monitor code
local spawnchance = {};
local numchoices = 0;

freeslot("SPR_RKEL", "SPR_RKMS");

-- Handles random monitor weights via console.
local function QUESTIONBOXCHANCES(objtype, cvar)
    local i = cvar.value; -- pain time

    while (i) 
    do
        spawnchance[numchoices] = objtype;
        numchoices = $ + 1;
        i = $ - 1;
    end
end


local function P_DoRandomBoxChances(void)
	if (not (netgame or multiplayer))
		local rand = P_RandomKey(10);

        if (rand == 0)
			return MT_RING_ICON;
		elseif (rand == 1)
			return MT_SNEAKERS_ICON;
        elseif (rand == 2)
			return MT_INVULN_ICON;
        elseif (rand == 3)
			return MT_WHIRLWIND_ICON;
        elseif (rand == 4)
			return MT_ELEMENTAL_ICON;
        elseif (rand == 5)
			return MT_ATTRACT_ICON;
        elseif (rand == 6)
			return MT_FORCE_ICON;
        elseif (rand == 7)
			return MT_ARMAGEDDON_ICON;
        elseif (rand == 8)
			return MT_1UP_ICON;
		elseif (rand == 9)
			return MT_EGGMAN_ICON;
        end

		return MT_NULL;
	end

    spawnchance = {};
    numchoices = 0;
	
    QUESTIONBOXCHANCES(MT_RING_ICON,       CV_FindVar("tv_superring"));
    QUESTIONBOXCHANCES(MT_SNEAKERS_ICON,   CV_FindVar("tv_supersneaker")); -- THIS HAS A DIFFERENT NAME WHY
    QUESTIONBOXCHANCES(MT_INVULN_ICON,     CV_FindVar("tv_invincibility"));
    QUESTIONBOXCHANCES(MT_WHIRLWIND_ICON,  CV_FindVar("tv_jumpshield"));
    QUESTIONBOXCHANCES(MT_ELEMENTAL_ICON,  CV_FindVar("tv_watershield"));
    QUESTIONBOXCHANCES(MT_ATTRACT_ICON,    CV_FindVar("tv_ringshield"));
    QUESTIONBOXCHANCES(MT_FORCE_ICON,      CV_FindVar("tv_forceshield"));
    QUESTIONBOXCHANCES(MT_ARMAGEDDON_ICON, CV_FindVar("tv_bombshield"));
    QUESTIONBOXCHANCES(MT_1UP_ICON,        CV_FindVar("tv_1up"));
    QUESTIONBOXCHANCES(MT_EGGMAN_ICON,     CV_FindVar("tv_eggman"));
    QUESTIONBOXCHANCES(MT_MIXUP_ICON,      CV_FindVar("tv_teleporter"));
    QUESTIONBOXCHANCES(MT_RECYCLER_ICON,   CV_FindVar("tv_recycler"));

	if (numchoices == 0) then return MT_NULL; end
	return spawnchance[P_RandomKey(numchoices)];
end

function A_MonitorPop(actor)
	local item = 0;
	local newmobj;

	-- Spawn the "pop" explosion.
	if (actor.info.deathsound)
		S_StartSound(actor, actor.info.deathsound);
	end
	
	P_SpawnMobjFromMobj(actor, 0, 0, actor.height/4, MT_EXPLODE);

	-- We're dead now. De-solidify.
	actor.health = 0;
	actor.flags = ($ & ~MF_SOLID);
	actor.flags = $|MF_NOCLIP;

	if (actor.info.damage == MT_UNKNOWN)
		-- MT_UNKNOWN is random. Because it's unknown to us... get it?
		item = P_DoRandomBoxChances();

		if (item == MT_NULL)
			return;
		end
	else
		item = actor.info.damage;
	end

	if (item == 0)
		return;
	end
	
	if (actor.target.skin == "sgimario")
		if (item == MT_1UP_ICON)
			if (actor.tracer) // Remove the old lives icon.
				P_RemoveMobj(actor.tracer);
			end
			S_StartSound(newmobj, sfx_641sp);
			
			newmobj = P_SpawnMobjFromMobj(actor, 0, 0, 13*FRACUNIT, MT_THOK);
			newmobj.tics = -1;
			newmobj.fuse = -1;
			newmobj.tracer = actor.target;
			newmobj.sprite = SPR_RKEL;
			newmobj.frame = ((ultimatemode) and B or A);
			newmobj.spawnanim = 1;
			
			newmobj.sx = newmobj.x;
			newmobj.sy = newmobj.y;
			newmobj.sz = newmobj.z;
			return;
		elseif (item == MT_INVULN_ICON)
			newmobj = P_SpawnMobjFromMobj(actor, 0, 0, 13*FRACUNIT, MT_INVINCSTAR);
			newmobj.tracer = actor.target;
			return;
		end
	end
	
	newmobj = P_SpawnMobjFromMobj(actor, 0, 0, 13*FRACUNIT, item);
	newmobj.target = actor.target; // Transfer target

	if (item == MT_1UP_ICON)
		if (actor.tracer) // Remove the old lives icon.
			P_RemoveMobj(actor.tracer);
			
		end
		
		if (not newmobj.target
		or not newmobj.target.player
		or not newmobj.target.skin
		or skins[newmobj.target.skin].sprites[SPR2_LIFE].numframes == 0)
			// No lives icon for this player, use the default.
		else
			// Spawn the lives icon.
			local livesico = P_SpawnMobjFromMobj(newmobj, 0, 0, 0, MT_OVERLAY);
			livesico.target = newmobj;
			newmobj.tracer = livesico;

			livesico.color = newmobj.target.player.mo.color;
			livesico.skin = newmobj.target.player.mo.skin;
			livesico.state = newmobj.info.seestate;

			// We're using the overlay, so use the overlay 1up sprite (no text)
			newmobj.sprite = SPR_TV1P;
		end
	end

	-- Run a linedef executor immediately upon popping
	-- You may want to delay your effects by 18 tics to sync with the reward giving
	if (actor.spawnpoint and actor.spawnpoint.args[0])
		P_LinedefExecute(actor.spawnpoint.args[0], actor.target, nil);
	end
end

function A_GoldMonitorPop(actor)
	local item = 0;
	local newmobj;

	// Don't spawn the "pop" explosion, because the monitor isn't broken.
	if (actor.info.deathsound)
		S_StartSound(actor, actor.info.deathsound);
	end

	// Remove our flags for a bit.
	// Players can now stand on top of us.
	actor.flags  = ($ & ~(MF_MONITOR|MF_SHOOTABLE));

	// Don't count this box in statistics. Sorry.

	actor.fuse = 0; // Don't let the monitor code screw us up.

	if (actor.info.damage == MT_UNKNOWN)
		// MT_UNKNOWN is random. Because it's unknown to us... get it?
		item = P_DoRandomBoxChances();

		if (item == MT_NULL)
			return;
		end
	else
		item = actor.info.damage;
	end

	if (item == 0)
		return;
	end
	
	// N64 Mario behavior
	if (actor.target.skin == "sgimario")
		if (item == MT_1UP_ICON)
			if (actor.tracer) // Remove the old lives icon.
				P_RemoveMobj(actor.tracer);
			end
			S_StartSound(newmobj, sfx_641sp);
			
			newmobj = P_SpawnMobjFromMobj(actor, 0, 0, 14*FRACUNIT, MT_THOK);
			newmobj.tics = -1;
			newmobj.fuse = -1;
			newmobj.tracer = actor.target;
			newmobj.sprite = SPR_RKEL;
			newmobj.frame = A;
			newmobj.spawnanim = 1;
			
			newmobj.sx = newmobj.x;
			newmobj.sy = newmobj.y;
			newmobj.sz = newmobj.z;
			return;
		elseif (item == MT_INVULN_ICON)
			newmobj = P_SpawnMobjFromMobj(actor, 0, 0, actor.height+1, MT_INVINCSTAR);
			newmobj.tracer = actor.target;
			return;
		end
	end

	// Note: the icon spawns 1 fracunit higher
	newmobj = P_SpawnMobjFromMobj(actor, 0, 0, 14*FRACUNIT, item);
	newmobj.target = actor.target; // Transfer target

	if (item == MT_1UP_ICON)
		if (actor.tracer) // Remove the old lives icon.
			P_RemoveMobj(actor.tracer);
			
		end
		
		if (not newmobj.target
		or not newmobj.target.player
		or not newmobj.target.skin
		or skins[newmobj.target.skin].sprites[SPR2_LIFE].numframes == 0)
			// No lives icon for this player, use the default.
		else
			// Spawn the lives icon.
			local livesico = P_SpawnMobjFromMobj(newmobj, 0, 0, 0, MT_OVERLAY);
			livesico.target = newmobj;
			newmobj.tracer = livesico;

			livesico.color = newmobj.target.player.mo.color;
			livesico.skin = newmobj.target.player.mo.skin;
			livesico.state = newmobj.info.seestate;

			// We're using the overlay, so use the overlay 1up sprite (no text)
			newmobj.sprite = SPR_TV1P;
		end
	end

	-- Run a linedef executor immediately upon popping
	-- You may want to delay your effects by 18 tics to sync with the reward giving
	if (actor.spawnpoint and actor.spawnpoint.args[0])
		P_LinedefExecute(actor.spawnpoint.args[0], actor.target, nil);
	end
end


// Function: A_RingBox
//
// Description: Awards the player 10 rings.
//
// var1 = unused
// var2 = unused
//
function A_RingBox(actor)
	local player;

	if ((not actor.target) or (not actor.target.player))
		return;
	end

	player = actor.target.player;

	P_GivePlayerRings(player, actor.info.reactiontime);
	
	if (player.mo.skin == "sgimario")
	and player.rings > 9989
		player.rk_maxcoins = $ + actor.info.reactiontime;
	end
	
	if (actor.info.seesound)
		S_StartSound(player.mo, actor.info.seesound);
	end
end

function A_64ExtraLife(actor)
	
	if (N64MarioLocal() and (not multiplayer))
		actor.frame = 1;
	end
end

function A_64Invinc(actor)
	
	if (N64MarioLocal() and (not multiplayer))
		actor.sprite = SPR_RKMS;
	else
		actor.sprite = SPR_TVIV;
	end
end

states[S_PLAY_BOX1].action = A_64ExtraLife
states[S_PLAY_ICON1].action = A_64ExtraLife
states[S_PLAY_ICON3].action = A_64ExtraLife

states[S_INVULN_BOX].action = A_64Invinc
states[S_INVULN_GOLDBOX].action = A_64Invinc
