-- LUA_RKYANIM - N64 Mario Animation Controller
-- these are functions and data related to a custom animation system for N64 mario
-- animations use "frame banks" (tails' tail sprites, from TAL3 to TALC) and arrays to handle everything
-- custom states are also utilized
-- animation data type: 0x00BBAAAA

rawset(_G, "rkymariodata", {
	animBanks = nil,
    animData = nil, -- 2D array with databanks
})

-- you see, I'd have done this in the rawset constructor, but WARNING: ...\LUA_RKYANIM.lua:11: '}' 
-- expected (to close '{' at line 7) near '='
-- therefore, you get this crap instead

-- set up bank sprites
rkymariodata.animBanks = {SPR2_TAL3, SPR2_TAL4, SPR2_TAL5, SPR2_TAL6, SPR2_TAL7,
                          SPR2_TAL8, SPR2_TAL9, SPR2_TALA, SPR2_TALB, SPR2_TALC};
						  
-- set up data arrays
rkymariodata.animData = {};

for i = 1, 10
do
	rkymariodata.animData[i] = {};
end

-- animation data flags
local RAF_SETDELAY = 0x010000;
local RAF_ENDANIM = 0xFF0000;
local RAF_ENDANMKEEPSTATE = 0xFE0000;

-- RKA_SIDESOMER, bank 1, anim 1
rkymariodata.animData[1][1] = { (RAF_SETDELAY|2), A, B, C, D, E, F, G, H, I, J, RAF_ENDANMKEEPSTATE};
rkymariodata.animData[1][2] = { (RAF_SETDELAY|2), K, L, M, N, O, P, Q, R, S, T, RAF_ENDANMKEEPSTATE};
rkymariodata.animData[1][3] = { (RAF_SETDELAY|4), A, B, C, D, E, F, G, H};

rawset(_G, "RKY_StartCustomAnim", function(mo, data)
	local bank = ((data >> 16) & 0xFF);
	local anim = (data & 0xFFFF);
	local state = max(0, bank - 1);
	
	-- set animation stuff
	mo.rkyAnimBank = bank;
	mo.rkyAnimNum = anim;
	
	-- set animation state
	state = $ << 16;
	mo.rkyExtraState = (state|anim);
	mo.state = S_PLAY_RANM;
	
	mo.rkyAnimFrame = 1;
	mo.rkyAnimDelay = 1;
	mo.rkyAnimFrameTime = 0;
end)

rawset(_G, "RKY_HandleCustomAnim", function(mo)
	-- here comes the prelim check
	
	-- clear custom animation stuff if we're not in a dedicated state
	if (mo.state ~= S_PLAY_RANM)
	then
		mo.rkyExtraState = nil;
		mo.rkyAnimBank = nil;
		mo.rkyAnimNum = nil;
		mo.rkyAnimFrame = nil;
		mo.rkyAnimDelay = nil;
		mo.rkyAnimFrameTime = nil;
	end
	
	-- if we have no data, don't do anything
	if ((mo.rkyAnimBank == nil) or (mo.rkyAnimNum == nil) or (mo.rkyExtraState == nil)
	     or (mo.rkyAnimFrame == nil) or (mo.rkyAnimDelay == nil) or (mo.rkyAnimFrameTime == nil))
	then
		return;
	end
	
	local bnk = mo.rkyAnimBank;
	local anm = mo.rkyAnimNum;
	local framedata = rkymariodata.animData[bnk][anm][mo.rkyAnimFrame];
	local animlen = #rkymariodata.animData[bnk][anm];
	local flag = 0;
	
	mo.sprite2 = rkymariodata.animBanks[bnk];
	
	if (framedata == nil)
	then
		--print("FRAMEDATA IS NIL!!");
		--local str = "bank: "..bnk..", anim: "..anm..", frame: "..mo.rkyAnimFrame;
		--print(str);
		--print(rkymariodata.animData[1][1]);
		return;
	end
	
	if (framedata & 0xFF0000) // we've got a dataflag
	then
		flag = (framedata >> 16);
		
		if (flag == 1) -- Set Delay
		then
			--local dstr = "delay is now "..(framedata & 0xFFFF);
			--print(dstr);
			mo.rkyAnimDelay = max(1, (framedata & 0xFFFF)); // frame data can be from 1 to 65535
		elseif ((flag == 0xFE) or (flag == 255)) -- End Animation
		then
			-- clear all custom animation data
			--print("anim end");
			
			-- only clear the state if it's a total wipe
			if (flag == 255)
			then
				mo.rkyExtraState = nil;
			end
			
			mo.rkyAnimBank = nil;
			mo.rkyAnimNum = nil;
			mo.rkyAnimFrame = nil;
			mo.rkyAnimDelay = nil;
			mo.rkyAnimFrameTime = nil;
			
			-- return immediately
			return;
		end
		
		-- jump right to the next frame
		mo.rkyAnimFrame = ($ + 1);
		
		framedata = rkymariodata.animData[bnk][anm][mo.rkyAnimFrame];
		
		if (framedata == nil)
		then
			mo.rkyAnimFrame = 1;
		end
	end
	
	mo.frame = framedata;
	
	-- setting speed to 0 freezes the animation
	if (mo.rkyAnimDelay > 0)
	then
		mo.rkyAnimFrameTime = $ + 1;
		--print(mo.rkyAnimFrameTime);
	end
	
	if (mo.rkyAnimFrameTime >= mo.rkyAnimDelay)
	then
		--print("frame increment");
		mo.rkyAnimFrame = ($ + 1);

		if (rkymariodata.animData[bnk][anm][mo.rkyAnimFrame] == nil)
		then
			mo.rkyAnimFrame = 1;
		end
		
		mo.rkyAnimFrameTime = 0;
	end
end)