-- LUA_STAR
-- another decomp conversion
-- like the lib64 stuff, this code IS reusable
freeslot("sfx_64ens", "sfx_64gsa", "SPR_RKST", "S_ROKUSTAR", "MT_INVINCSTAR");

states[S_ROKUSTAR] = {SPR_RKST, FF_FULLBRIGHT|A, -1, nil, 0, 0, S_ROKUSTAR};

mobjinfo[MT_INVINCSTAR] = {
	doomednum = -1,
	spawnstate = S_ROKUSTAR,
	spawnhealth = 1000,
	radius = 20*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOCLIPTHING|MF_NOGRAVITY
}


local SOUND_ENV_STAR = sfx_64ens;
local SOUND_GENERAL_STAR_APPEARS = sfx_64gsa;
local invulntics = 20*TICRATE;

local function cur_obj_become_tangible(o)
    if (o.flags & MF_NOCLIP)
        o.flags = ($ & ~MF_NOCLIP);
    end

    if (o.flags & MF_NOCLIPTHING)
        o.flags = ($ & ~MF_NOCLIPTHING)
    end

    o.oIntangibleTimer = 0;
end

local function P_MakeTracerInvincible(o)
    if (o.tracer and o.tracer.player)
        o.tracer.player.powers[pw_invulnerability] = invulntics + 1;
        P_PlayJingle(o.tracer.player, ((mariomode) and JT_MINV or JT_INV));
    end
end

local function P_SpawnStarSparkle(mo)
	local frac = FRACUNIT/4;
	
    local sparkle = P_SpawnMobj(mo.x + FixedMul(FixedMul(P_RandomRange(-frac,frac), 180*mo.scale), cos(mo.angle)),
								mo.y + FixedMul(FixedMul(P_RandomRange(-frac,frac), 180*mo.scale), sin(mo.angle)),
								mo.z + FixedMul(P_RandomRange(-frac,frac), 180*mo.scale),
								MT_THOK)
	sparkle.tics = -1;
	sparkle.fuse = -1;
	
	sparkle.frame = A;
	sparkle.state = S_RKSTRSP1;
	
    if (sparkle ~= nil)
        sparkle.scale = FixedMul(mo.scale, P_RandomRange(FRACUNIT-1, 0));
    end
end

local function P_SpawnStarMultiSparkle(mo, amt)
	for i = 0, amt
		P_SpawnStarSparkle(mo);
	end
end

local function bhv_star_spawn_loop(o)
    if (o.oTimer ~= nil)
		o.oTimer = $ + 1;
    end
    
    if (o.oAction ~= nil)
        if (o.oAction == 0)
            o.angle = $ + 0x1000*FRACUNIT;
            if (o.oTimer > 20)
                o.oTimer = 0;
				o.oAction = 2;
            end

        elseif (o.oAction == 1) -- this gets skipped over, but uhhhhhhhh
            o.oStarSpawnUnkFC = $ + o.momz;

            local sinang = ((o.oTimer * 0x8000) / 30)*FRACUNIT;

            o.z = o.oStarSpawnUnkFC + FixedMul(sin(FixedAngle(sinang)), 400*FRACUNIT);
            o.angle = $ + 0x1000*FRACUNIT;
            //P_SpawnStarSparkle(o);

            if (o.oTimer % 3 == 0)
                S_StartSound(o, SOUND_ENV_STAR);
            end

            if (o.oTimer == 30)
                o.oTimer = 0;
				o.oAction = 2;
                o.momx = 0;
                o.momy = 0;
            end

        elseif (o.oAction == 2)
			if (o.oTimer < 20)
                o.momz = ((20 - o.oTimer)*(o.scale/2))*P_MobjFlip(o);
            else
                o.momz = ((-10)*(o.scale/2))*P_MobjFlip(o);
            end

            P_SpawnStarMultiSparkle(o, 2);

            o.angle = $ - (o.oTimer * 0x10 + 0x1000)*FRACUNIT;
            if (o.oTimer % 3 == 0)
                S_StartSound(o, SOUND_ENV_STAR);
            end
            
            if (P_MobjFlip(o) >= 0)
                if (o.z < o.oHomeZ)
                    S_StopSoundByID(o, SOUND_ENV_STAR);
                    S_StartSound(o, SOUND_GENERAL_STAR_APPEARS);
                    //cur_obj_become_tangible(o);
                    o.momz = 0;
                    o.z = o.oHomeZ;

                    P_MakeTracerInvincible(o);
                    
                    o.oAction = 3;
                end
            else
                if (o.z > o.oHomeZ)
                    S_StopSoundByID(o, SOUND_ENV_STAR);
                    S_StartSound(o, SOUND_GENERAL_STAR_APPEARS);
                    //cur_obj_become_tangible(o);
                    o.momz = 0;
                    o.z = o.oHomeZ;

                    P_MakeTracerInvincible(o);
                    
                    o.oAction = 3;
                end
            end

        elseif (o.oAction == 3)
            o.angle = $ + 0x800*FRACUNIT;
            if (not o.despawnTime)
                o.despawnTime = o.oTimer + TICRATE;
            elseif (o.oTimer >= o.despawnTime)
                P_RemoveMobj(o);
                return;
            end
			
			P_SpawnStarMultiSparkle(o, 2); -- not actually present in source, but putting it here for the "fly away" anim
			
            o.momz = min(15*o.scale, abs($) + 2*(o.scale))*P_MobjFlip(o);
        end
    end
end

addHook("MobjCollide", function(mo, tm)
    if (not mo.oIntangibleTimer)
        if (mo.z > tm.z + tm.height)
            return true; // overhead
        end

        if (mo.z + mo.height < tm.z)
            return true; // underneath
        end
    end
end, MT_INVINCSTAR)

-- SM64 data setup
addHook("MobjSpawn", function(mo)
    mo.oAction = 0;
    mo.oTimer = 15;
    mo.oHomeZ = mo.z; -- get spawnpoint Z
	mo.oStarSpawnUnkFC = mo.oHomeZ;
end, MT_INVINCSTAR)

addHook("MobjThinker", bhv_star_spawn_loop, MT_INVINCSTAR)